@@ -758,6 +758,12 @@ namespace apg {
         return rep;
     }
 
+    std::string wechslernone(std::vector<bitworld> &bwv, int64_t *rect) {
+
+        return canonise_orientation(bwv, rect[2], rect[3], rect[0], rect[1],  1,  0,  0,  1);
+
+    }
+
     std::string wechslerise(std::vector<bitworld> &bwv, int64_t *rect) {
 
         std::string rep = "#";
