@@ -204,6 +204,11 @@ class Pattern(object):
         newptr = self.lifelib('TransformPattern', self.ptr, tfm)
         return Pattern(self.session, newptr, self.owner)
 
+    def __copy__(self):
+
+        newptr = self.lifelib('CopyPattern', self.ptr)
+        return Pattern(self.session, newptr, self.owner)
+
     def shift(self, x, y, exponent=0):
 
         if not isinstance(x, Integral):
