@@ -14,6 +14,12 @@ class TestGoL(unittest.TestCase):
         self.sess = lifelib.load_rules('b3s23')
         self.lt = self.sess.lifetree(memory=1000, n_layers=4)
 
+    def test_ulimit(self):
+
+        self.sess.lifelib.ensure_limit(10000, 1)
+        self.sess.lifelib.ensure_limit(20000, 1)
+        self.sess.lifelib.ensure_limit(10000, 1)
+
     def test_components(self):
 
         pat = self.lt.pattern('''7b3o5b3o16b3o5b3o$6bo2bo5bo2bo15bo2bo3bo2bo$6bo3bo3bo3bo15bo9bo$7b4o3b
@@ -30,6 +36,17 @@ $3b3o3b2o3b2o3bo2bo7bo2bo12b3o$3bo2bo2b2o3b2o2bo3bo7bob2o12bo2bo$3bo
         ncomps = len(pat.components())
         self.assertEqual(ncomps, 41)
 
+    def test_copy(self):
+
+        pat = self.lt.pattern('xp40_33zxs48g69gvzx3021zy833')
+        pat2 = pat.__copy__()
+
+        self.assertEqual(pat, pat2)
+        pat += pat(1093, 3511)
+        self.assertNotEqual(pat, pat2)
+        pat2 += pat2(1093, 3511)
+        self.assertEqual(pat, pat2)
+
     def test_samples(self):
 
         pat = self.lt.pattern('xp40_33zxs48g69gvzx3021zy833')
