@@ -2,7 +2,7 @@ import unittest
 import lifelib
 import os
 
-oca_rules = ['g4b2ikn3ais345', 'b36s23', 'g3b3s23', 'r7b65t95s65t114', 'g11b3s23', 'r2b1e7108s3e7c80']
+oca_rules = ['g4b2ikn3ais345', 'b36s23', 'g3b3s23', 'r7b65t95s65t114', 'g11b3s23', 'r2b1e7108s3e7c80', 'b347s3456f038k234567l0346']
 
 class TestOtherRules(unittest.TestCase):
 
@@ -15,6 +15,13 @@ class TestOtherRules(unittest.TestCase):
 
         self.sess = lifelib.load_rules(*oca_rules)
 
+    def test_bsfkl(self):
+
+        lt = self.sess.lifetree()
+        apgcode = 'xq20_xeefz77_y2g5zw55'
+        ss = lt.pattern(apgcode, 'b347s3456f038k234567l0346')
+        self.assertEqual(ss.period, 20)
+
     def test_uint64(self):
 
         try:
